@@ -1,4 +1,3 @@
-from __future__ import division, print_function, absolute_import
 import numpy as np
 from scipy.interpolate import interp1d
 try:
@@ -7,7 +6,7 @@ except ImportError:
     from collections import Iterable
 from copy import deepcopy as make_copy
 from scipy.ndimage import binary_dilation
-from .config import BIG_NUMBER, MIN_LOG, MIN_INTEGRATION_PEAK, TINY_NUMBER
+from .config import BIG_NUMBER, MIN_LOG, MIN_INTEGRATION_PEAK, TINY_NUMBER, SUPERTINY_NUMBER
 from .utils import clip
 
 class Distribution(object):
@@ -98,13 +97,23 @@ class Distribution(object):
             new_xmax = np.min([k.xmax for k in dists])
 
             x_vals = np.unique(np.concatenate([k.x for k in dists]))
-            x_vals = x_vals[(x_vals>new_xmin-TINY_NUMBER)&(x_vals<new_xmax+TINY_NUMBER)]
+            x_vals = x_vals[(x_vals> new_xmin-TINY_NUMBER)&(x_vals< new_xmax+TINY_NUMBER)]
             y_vals = np.sum([k.__call__(x_vals) for k in dists], axis=0)
-            peak = y_vals.min()
+            try:
+                peak = y_vals.min()
+            except:
+                print("WARNING: Unexpected behavior detected in multiply function,"
+                        "if you see this error \n please let us know by filling an issue at: https://github.com/neherlab/treetime/issues")
+                x_vals = [0,1]
+                y_vals = [BIG_NUMBER,BIG_NUMBER]
+                res = Distribution(x_vals, y_vals, is_log=True,
+                                    min_width=min_width, kind='linear')
+                return res
             ind = (y_vals-peak)<BIG_NUMBER/1000
             n_points = ind.sum()
             if n_points == 0:
-                print ("ERROR in distribution multiplication: Distributions do not overlap")
+                print("WARNING: Unexpected behavior detected in multipy function,"
+                        "if you see this error \n please let us know by filling an issue at: https://github.com/neherlab/treetime/issues")
                 x_vals = [0,1]
                 y_vals = [BIG_NUMBER,BIG_NUMBER]
                 res = Distribution(x_vals, y_vals, is_log=True,
@@ -117,6 +126,39 @@ class Distribution(object):
 
         return res
 
+    @staticmethod
+    def divide(numerator, denominator):
+        '''
+        divides one distribution object by another. Note that this is in general an ill-defined procedure.
+        this is implemented here for the special case where the numerator is a product that contains the denominator
+        to produce a reduced product without the denominator.
+        '''
+        if numerator.is_delta or denominator.is_delta:
+            raise(ArithmeticError("Can not divide delta functions"))
+        dists = [numerator, denominator]
+        min_width = np.max([k.min_width for k in dists])
+        new_xmin = np.max([k.xmin for k in dists])
+        new_xmax = np.min([k.xmax for k in dists])
+        x_vals = np.unique(np.concatenate([k.x for k in dists]))
+        x_vals = x_vals[(x_vals> new_xmin-TINY_NUMBER)&(x_vals< new_xmax+TINY_NUMBER)]
+        y_vals = numerator.__call__(x_vals) - denominator.__call__(x_vals)
+
+        peak = y_vals.min()
+        ind = (y_vals-peak)<BIG_NUMBER/1000
+        n_points = ind.sum()
+        if n_points == 0:
+            print("WARNING: Unexpected behavior detected in multipy function,"
+                    "if you see this error \n please let us know by filling an issue at: https://github.com/neherlab/treetime/issues")
+            x_vals = [0,1]
+            y_vals = [BIG_NUMBER,BIG_NUMBER]
+            res = Distribution(x_vals, y_vals, is_log=True,
+                                min_width=min_width, kind='linear')
+        elif n_points == 1:
+            res = Distribution.delta_function(x_vals[0])
+        else:
+            res = Distribution(x_vals[ind], y_vals[ind], is_log=True,
+                                min_width=min_width, kind='linear', assume_sorted=True)
+        return res
 
     def __init__(self, x, y, is_log=True, min_width = MIN_INTEGRATION_PEAK,
                  kind='linear', assume_sorted=False):
@@ -153,6 +195,8 @@ class Distribution(object):
             self._func= interp1d(xvals, yvals, kind=kind, fill_value=BIG_NUMBER,
                                  bounds_error=False, assume_sorted=True)
             self._fwhm = Distribution.calc_fwhm(self)
+            # remember effective range
+            self._effective_support = self.calc_effective_support()
 
         elif np.isscalar(x):
             assert (np.isscalar(y) or y is None)
@@ -167,6 +211,7 @@ class Distribution(object):
             self._xmin, self._xmax = x, x
             self._support = 0.
             self._func = lambda x : (x==self.peak_pos)*self.peak_val
+            self._effective_support = (self._xmin, self._xmax)
         else:
             raise TypeError("Cannot create Distribution: "
                 "Input arguments should be scalars or iterables!")
@@ -200,6 +245,10 @@ class Distribution(object):
     def fwhm(self):
         return self._fwhm
 
+    @property
+    def effective_support(self):
+        return self._effective_support
+
     @property
     def x(self):
         if self.is_delta:
@@ -247,28 +296,60 @@ class Distribution(object):
     def __mul__(self, other):
         return Distribution.multiply((self, other))
 
+    def calc_effective_support(self, cutoff=1e-15):
+        """
+        Assess the interval on which the value of self is higher than cutoff
+        relative to its peak
+        """
+        from scipy.optimize import brentq
+        log_cutoff = -np.log(cutoff)
+        vals = log_cutoff - self.__call__(self.x) + self.peak_val
+        above = vals > 0
+        above_idx = np.where(above)[0]
+        if len(above_idx)==0:
+            return (self.xmin, self.xmax)
+
+        try:
+            if above[0]:
+                left = self.xmin
+            else:
+                x1, x2 = self.x[above_idx[0]-1], self.x[above_idx[0]]
+                y1, y2 = vals[above_idx[0]-1], vals[above_idx[0]]
+                d = y2-y1
+                left = x1*y2/d - x2*y1/d
+
+            if above[-1]:
+                right = self.xmax
+            else:
+                x1, x2 = self.x[above_idx[-1]], self.x[above_idx[-1]+1]
+                y1, y2 = vals[above_idx[-1]], vals[above_idx[-1]+1]
+                d = y1-y2
+                right = -x1*y2/d + x2*y1/d
+        except:
+            raise ArithmeticError("Region of support of the distribution could not be determined!")
+
+        return (left,right)
+
 
     def _adjust_grid(self, rel_tol=0.01, yc=10):
-        updated = True
         n_iter=0
-        while len(self.y)>200 and updated and n_iter<5:
+        while len(self.y)>200 and n_iter<5:
             interp_err = 2*self.y[1:-1] - self.y[2:] - self.y[:-2]
             ind = np.ones_like(self.y, dtype=bool)
             dy = self.y-self.peak_val
             prune = interp_err[::2] > rel_tol*(1+ (dy[1:-1:2]/yc)**4)
             ind[1:-1:2] = prune
+            ind[self.peak_idx] = True
             if np.mean(prune)<1.0:
                 self._func.y = self._func.y[ind]
                 self._func.x = self._func.x[ind]
-                updated=True
                 n_iter+=1
             else:
-                updated=False
-                n_iter+=1
+                break
 
-        self._peak_idx = self.__call__(self._func.x).argmin()
-        self._peak_pos = self._func.x[self._peak_idx]
-        self._peak_val = self.__call__(self.peak_pos)
+            self._peak_idx = self.__call__(self._func.x).argmin()
+            self._peak_pos = self._func.x[self._peak_idx]
+            self._peak_val = self.__call__(self.peak_pos)
 
 
     def prob(self,x):
@@ -283,12 +364,16 @@ class Distribution(object):
         if factor>=0:
             self._xmin*=factor
             self._xmax*=factor
+            self._fwhm*=factor
+            self._effective_support = [x*factor for x in self._effective_support]
         else:
             tmp = self.xmin
             self._xmin = factor*self.xmax
             self._xmax = factor*tmp
             self._func.x = self._func.x[::-1]
             self._func.y = self._func.y[::-1]
+            self._fwhm *= -factor
+            self._effective_support = [x*factor for x in self._effective_support[::-1]]
 
 
     def integrate(self, return_log=False ,**kwargs):
@@ -334,3 +419,16 @@ class Distribution(object):
 
         return np.sum(res)
 
+
+    def fft(self, T, n=None, inverse_time=True):
+        if self.is_delta:
+            import ipdb; ipdb.set_trace()
+        from numpy.fft import rfft
+        if n is None:
+            n=len(T)
+        if inverse_time:
+            return rfft(self.prob_relative(T), n=n)
+        else:
+            return rfft(self.prob_relative(T)[::-1], n=n)
+
+
