@@ -1,9 +1,8 @@
-from __future__ import print_function, division, absolute_import
 import numpy as np
 from scipy import optimize as sciopt
 from Bio import Phylo
 from . import config as ttconf
-from . import MissingDataError,UnknownMethodError,NotReadyError
+from . import MissingDataError,UnknownMethodError,NotReadyError,TreeTimeError
 from .utils import tree_layout
 from .clock_tree import ClockTree
 
@@ -11,6 +10,24 @@ rerooting_mechanisms = ["min_dev", "best", "least-squares"]
 deprecated_rerooting_mechanisms = {"residual":"least-squares", "res":"least-squares",
                                    "min_dev_ML": "min_dev", "ML":"least-squares"}
 
+
+def reduce_time_marginal_argument(input_time_marginal):
+    '''
+    This function maps deprecated arguments/terms for the timetree inference mode
+    to recommended terms.
+    '''
+    if input_time_marginal in [False, 'false', 'never']:
+        return 'never'
+    elif input_time_marginal in [True, 'always', 'true']:
+        return 'always'
+    elif input_time_marginal in ['only-final', 'assign']:
+        return 'only-final'
+    elif input_time_marginal == 'confidence-only':
+        return input_time_marginal
+    else:
+        raise ValueError(f"'{input_time_marginal}' is not a known time marginal argument")
+
+
 class TreeTime(ClockTree):
     """
     TreeTime is a wrapper class to ClockTree that adds additional functionality
@@ -36,8 +53,9 @@ class TreeTime(ClockTree):
 
     def run(self, root=None, infer_gtr=True, relaxed_clock=None, n_iqd = None,
             resolve_polytomies=True, max_iter=0, Tc=None, fixed_clock_rate=None,
-            time_marginal=False, sequence_marginal=False, branch_length_mode='auto',
-            vary_rate=False, use_covariation=False, tracelog_file=None, **kwargs):
+            time_marginal='never', sequence_marginal=False, branch_length_mode='auto',
+            vary_rate=False, use_covariation=False, tracelog_file=None,
+            method_anc = 'probabilistic', assign_gamma=None, **kwargs):
 
         """
         Run TreeTime reconstruction. Based on the input parameters, it divides
@@ -57,7 +75,7 @@ class TreeTime(ClockTree):
         infer_gtr : bool
            If True, infer GTR model
 
-        relaxed_clock : dic
+        relaxed_clock : dict
            If not None, use autocorrelated molecular clock model. Specify the
            clock parameters as :code:`{slack:<slack>, coupling:<coupling>}` dictionary.
 
@@ -85,8 +103,11 @@ class TreeTime(ClockTree):
         fixed_clock_rate : float
            Fixed clock rate to be used. If None, infer clock rate from the molecular clock.
 
-        time_marginal : bool
-           If True, perform a final round of marginal reconstruction of the node's positions.
+        time_marginal : bool, str
+           If False perform joint reconstruction of the divergence times, if True use marginal
+           reconstruction of the divergence times, if 'only_final' (or 'assign') apply the marginal reconstruction
+           only to the last optimization round, if "confidence-only" perform additional round using marginal
+           reconstruction for calculation of confidence intervals but do not update times.
 
         sequence_marginal : bool, optional
             use marginal reconstruction for ancestral sequences
@@ -109,6 +130,15 @@ class TreeTime(ClockTree):
             regression ignoring phylogenetic covaration between nodes. If vary_rate is True,
             use_covariation is true by default
 
+        method_anc: str, optional
+            Which method should be used to reconstruct ancestral sequences.
+            Supported values are "parsimony", "fitch", "probabilistic" and "ml".
+            Default is "probabilistic"
+
+        assign_gamma: callable, optional
+            function to specify gamma (branch length scaling, local clock rate modifier)
+            for each branch in tree, not compatible with a relaxed clock model
+
         **kwargs
            Keyword arguments needed by the downstream functions
 
@@ -120,7 +150,6 @@ class TreeTime(ClockTree):
 
 
         """
-
         # register the specified covaration mode
         self.use_covariation = use_covariation or (vary_rate and (not type(vary_rate)==float))
 
@@ -128,16 +157,17 @@ class TreeTime(ClockTree):
             raise MissingDataError("TreeTime.run: ERROR, alignment or tree are missing")
         if self.aln is None:
             branch_length_mode='input'
-
         self._set_branch_length_mode(branch_length_mode)
 
         # determine how to reconstruct and sample sequences
         seq_kwargs = {"marginal_sequences":sequence_marginal or (self.branch_length_mode=='marginal'),
                       "branch_length_mode": self.branch_length_mode,
-                      "sample_from_profile":"root",
+                      "sample_from_profile": "root",
+                      "prune_short":kwargs.get("prune_short", True),
                       "reconstruct_tip_states":kwargs.get("reconstruct_tip_states", False)}
-
-        tt_kwargs = {'clock_rate':fixed_clock_rate, 'time_marginal':False}
+        time_marginal_method = reduce_time_marginal_argument(time_marginal) ## for backward compatibility
+        tt_kwargs = {'clock_rate':fixed_clock_rate,
+                     'time_marginal':False if time_marginal_method in ['never', 'only-final', 'confidence-only'] else True}
         tt_kwargs.update(kwargs)
 
         seq_LH = 0
@@ -146,15 +176,18 @@ class TreeTime(ClockTree):
         if "do_marginal" in kwargs:
             time_marginal=kwargs["do_marginal"]
 
+        if assign_gamma and relaxed_clock:
+            raise TreeTimeError("assign_gamma and relaxed clock are incompatible arguments")
+
         # initially, infer ancestral sequences and infer gtr model if desired
         if self.branch_length_mode=='input':
             if self.aln:
                 self.infer_ancestral_sequences(infer_gtr=infer_gtr, marginal=seq_kwargs["marginal_sequences"], **seq_kwargs)
-                self.prune_short_branches()
+                if seq_kwargs["prune_short"]:
+                    self.prune_short_branches()
         else:
             self.optimize_tree(infer_gtr=infer_gtr,
-                               max_iter=1, prune_short=True, **seq_kwargs)
-        avg_root_to_tip = np.mean([x.dist2root for x in self.tree.get_terminals()])
+                               max_iter=1, method_anc = method_anc, **seq_kwargs)
 
         # optionally reroot the tree either by oldest, best regression or with a specific leaf
         if n_iqd or root=='clock_filter':
@@ -171,7 +204,7 @@ class TreeTime(ClockTree):
             if self.aln:
                 self.infer_ancestral_sequences(**seq_kwargs)
         else:
-            self.optimize_tree(max_iter=1, prune_short=False,**seq_kwargs)
+            self.optimize_tree(max_iter=1, method_anc = method_anc, **seq_kwargs)
 
         # infer time tree and optionally resolve polytomies
         self.logger("###TreeTime.run: INITIAL ROUND",0)
@@ -179,11 +212,17 @@ class TreeTime(ClockTree):
 
         if self.aln:
             seq_LH = self.tree.sequence_marginal_LH if seq_kwargs['marginal_sequences'] else self.tree.sequence_joint_LH
-        self.LH =[[seq_LH, self.tree.positional_joint_LH, 0]]
+        self.LH =[[seq_LH, self.tree.positional_LH, 0]]
 
+        # if we reroot, repeat rerooting after initial clock-filter/time tree
+        # re-optimize branch length, and update time tree
         if root is not None and max_iter:
-            new_root = self.reroot(root='least-squares' if root=='clock_filter' else root, clock_rate=fixed_clock_rate)
+            self.reroot(root='least-squares' if root=='clock_filter' else root, clock_rate=fixed_clock_rate)
             self.logger("###TreeTime.run: rerunning timetree after rerooting",0)
+
+            if self.branch_length_mode!='input':
+                self.optimize_tree(max_iter=0, method_anc = method_anc,**seq_kwargs)
+
             self.make_time_tree(**tt_kwargs)
 
         # iteratively reconstruct ancestral sequences and re-infer
@@ -212,7 +251,6 @@ class TreeTime(ClockTree):
 
             # estimate a relaxed molecular clock
             if relaxed_clock:
-                print("relaxed_clock", relaxed_clock)
                 self.relaxed_clock(**relaxed_clock)
                 need_new_time_tree = True
 
@@ -221,11 +259,15 @@ class TreeTime(ClockTree):
                 # if polytomies are found, rerun the entire procedure
                 n_resolved = self.resolve_polytomies()
                 if n_resolved:
+                    seq_kwargs['prune_short']=False
                     self.prepare_tree()
                     if self.branch_length_mode!='input': # otherwise reoptimize branch length while preserving branches without mutations
-                        self.optimize_tree(prune_short=False, max_iter=0, **seq_kwargs)
-
+                        self.optimize_tree(max_iter=0, method_anc = method_anc,**seq_kwargs)
                     need_new_time_tree = True
+            if assign_gamma and callable(assign_gamma):
+                self.logger("### assigning gamma",1)
+                assign_gamma(self.tree)
+                need_new_time_tree = True
 
             if need_new_time_tree:
                 self.make_time_tree(**tt_kwargs)
@@ -240,7 +282,7 @@ class TreeTime(ClockTree):
 
             if self.aln:
                 seq_LH = self.tree.sequence_marginal_LH if seq_kwargs['marginal_sequences'] else self.tree.sequence_joint_LH
-            self.LH.append([seq_LH, self.tree.positional_joint_LH, self.tree.coalescent_joint_LH])
+            self.LH.append([seq_LH, self.tree.positional_LH, self.tree.coalescent_joint_LH])
 
             # Update the trace log
             self.trace_run.append(self.tracelog_run(niter=niter+1, ndiff=ndiff, n_resolved=n_resolved,
@@ -253,6 +295,7 @@ class TreeTime(ClockTree):
                 self.logger("###TreeTime.run: CONVERGED",0)
                 break
 
+
         # if the rate is too be varied and the rate estimate has a valid confidence interval
         # rerun the estimation for variations of the rate
         if vary_rate:
@@ -265,14 +308,14 @@ class TreeTime(ClockTree):
 
         # if marginal reconstruction requested, make one more round with marginal=True
         # this will set marginal_pos_LH, which to be used as error bar estimations
-        if time_marginal:
+        if time_marginal_method in ['only-final', 'confidence-only']:
             self.logger("###TreeTime.run: FINAL ROUND - confidence estimation via marginal reconstruction", 0)
-            tt_kwargs['time_marginal']=time_marginal
+            tt_kwargs['time_marginal'] = True
             self.make_time_tree(**tt_kwargs)
 
             self.trace_run.append(self.tracelog_run(niter=niter+1, ndiff=0, n_resolved=0,
-                                      time_marginal = tt_kwargs['time_marginal'],
-                                      sequence_marginal = seq_kwargs['marginal_sequences'], Tc=Tc, tracelog=tracelog_file))
+                                      time_marginal=tt_kwargs['time_marginal'],
+                                      sequence_marginal=seq_kwargs['marginal_sequences'], Tc=Tc, tracelog=tracelog_file))
 
         # explicitly print out which branches are bad and whose dates don't correspond to the input dates
         bad_branches =[n for n in self.tree.get_terminals()
@@ -474,6 +517,7 @@ class TreeTime(ClockTree):
             #(Without outgroup_branch_length, gives a trifurcating root, but this will mean
             #mutations may have to occur multiple times.)
             self.tree.root_with_outgroup(new_root, outgroup_branch_length=new_root.branch_length/2)
+            self.tree.root.clades.sort(key = lambda x:x.count_terminals())
             self.get_clock_model(covariation=use_cov, slope = slope)
 
 
@@ -501,7 +545,7 @@ class TreeTime(ClockTree):
         return new_root
 
 
-    def resolve_polytomies(self, merge_compressed=False):
+    def resolve_polytomies(self, merge_compressed=False, resolution_threshold=0.05):
         """
         Resolve the polytomies on the tree.
 
@@ -530,7 +574,7 @@ class TreeTime(ClockTree):
         for n in self.tree.find_clades():
             if len(n.clades) > 2:
                 prior_n_clades = len(n.clades)
-                self._poly(n, merge_compressed)
+                self._poly(n, merge_compressed, resolution_threshold=resolution_threshold)
                 poly_found+=prior_n_clades - len(n.clades)
 
         obsolete_nodes = [n for n in self.tree.find_clades() if len(n.clades)==1 and n.up is not None]
@@ -546,7 +590,7 @@ class TreeTime(ClockTree):
         return poly_found
 
 
-    def _poly(self, clade, merge_compressed):
+    def _poly(self, clade, merge_compressed, resolution_threshold):
 
         """
         Function to resolve polytomies for a given parent node. If the
@@ -554,7 +598,7 @@ class TreeTime(ClockTree):
         nothing. Otherwise, for each pair of nodes, assess the possible LH increase
         which could be gained by merging the two nodes. The increase in the LH is
         basically the tradeoff between the gain of the LH due to the changing the
-        branch lenghts towards the optimal values and the decrease due to the
+        branch lengths towards the optimal values and the decrease due to the
         introduction of the new branch with zero optimal length.
         """
 
@@ -567,8 +611,8 @@ class TreeTime(ClockTree):
             cost gain if nodes n1, n2 are joined and their parent is placed at time t
             cost gain = (LH loss now) - (LH loss when placed at time t)
             """
-            cg2 = n2.branch_length_interpolator(parent.time_before_present - n2.time_before_present) - n2.branch_length_interpolator(t - n2.time_before_present)
-            cg1 = n1.branch_length_interpolator(parent.time_before_present - n1.time_before_present) - n1.branch_length_interpolator(t - n1.time_before_present)
+            cg2 = n2.branch_length_interpolator._func(parent.time_before_present - n2.time_before_present) - n2.branch_length_interpolator._func(t - n2.time_before_present)
+            cg1 = n1.branch_length_interpolator._func(parent.time_before_present - n1.time_before_present) - n1.branch_length_interpolator._func(t - n1.time_before_present)
             cg_new = - zero_branch_slope * (parent.time_before_present - t) # loss in LH due to the new branch
             return -(cg2+cg1+cg_new)
 
@@ -600,19 +644,27 @@ class TreeTime(ClockTree):
                 # set zero to large negative value and find optimal pair
                 np.fill_diagonal(cost_gains, -1e11)
                 idxs = np.unravel_index(cost_gains.argmax(),cost_gains.shape)
-                if (idxs[0] == idxs[1]) or cost_gains.max()<0:
+                if (idxs[0] == idxs[1]) or cost_gains.max()<resolution_threshold:
                     self.logger("TreeTime._poly.merge_nodes: node is not fully resolved "+clade.name,4)
                     return LH
 
                 n1, n2 = source_arr[idxs[0]], source_arr[idxs[1]]
                 LH += cost_gains[idxs]
-
                 new_node = Phylo.BaseTree.Clade()
 
                 # fix positions and branch lengths
                 new_node.time_before_present = new_positions[idxs]
                 new_node.branch_length = clade.time_before_present - new_node.time_before_present
+                self.logger(f"TreeTime._poly.merge_nodes: merging {n1.name} and {n2.name}",4)
                 new_node.clades = [n1,n2]
+                if n1.mask is None or n2.mask is None:
+                    new_node.mask = None
+                    new_node.mcc = None
+                else:
+                    new_node.mask = n1.mask * n2.mask
+                    new_node.mcc = n1.mcc if n1.mcc==n2.mcc else None
+                    self.logger('TreeTime._poly.merge_nodes: assigning mcc to new node ' + new_node.mcc, 4)
+
                 n1.branch_length = new_node.time_before_present - n1.time_before_present
                 n2.branch_length = new_node.time_before_present - n2.time_before_present
 
@@ -632,8 +684,9 @@ class TreeTime(ClockTree):
                     new_node.profile_pair = self.marginal_branch_profile(new_node)
 
                 new_node.branch_length_interpolator = BranchLenInterpolator(new_node, self.gtr,
-                            pattern_multiplicity = self.data.multiplicity, min_width=self.min_width,
-                            one_mutation=self.one_mutation, branch_length_mode=self.branch_length_mode)
+                            pattern_multiplicity = self.data.multiplicity(mask=new_node.mask), min_width=self.min_width,
+                            one_mutation=self.one_mutation, branch_length_mode=self.branch_length_mode,
+                            n_grid_points = self.branch_grid_points)
 
                 clade.clades.remove(n1)
                 clade.clades.remove(n2)
@@ -668,7 +721,7 @@ class TreeTime(ClockTree):
         stretched = [c for c  in clade.clades if c.mutation_length < c.clock_length]
         compressed = [c for c in clade.clades if c not in stretched]
 
-        if len(stretched)==1 and merge_compressed is False:
+        if len(stretched)<2 and merge_compressed is False:
             return 0.0
 
         LH = merge_nodes(stretched, isall=len(stretched)==len(clade.clades))
@@ -691,13 +744,12 @@ class TreeTime(ClockTree):
         """
         try:
             u_lh = self.tree.unconstrained_sequence_LH
+            t_lh = self.tree.positional_LH
             if joint:
                 s_lh = self.tree.sequence_joint_LH
-                t_lh = self.tree.positional_joint_LH
                 c_lh = self.tree.coalescent_joint_LH
             else:
                 s_lh = self.tree.sequence_marginal_LH
-                t_lh = self.tree.positional_marginal_LH
                 c_lh = 0
 
             print ("###  Tree Log-Likelihood  ###\n"
@@ -710,7 +762,7 @@ class TreeTime(ClockTree):
             print("ERROR. Did you run the corresponding inference (joint/marginal)?")
 
 
-    def add_coalescent_model(self, Tc, **kwargs):
+    def add_coalescent_model(self, Tc, n_branches_posterior=False, **kwargs):
         """Add a coalescent model to the tree and optionally optimze
 
         Parameters
@@ -721,8 +773,8 @@ class TreeTime(ClockTree):
         """
         from .merger_models import Coalescent
         self.logger('TreeTime.run: adding coalescent prior with Tc='+str(Tc),1)
-        self.merger_model = Coalescent(self.tree,
-                                       date2dist=self.date2dist, logger=self.logger)
+        self.merger_model = Coalescent(self.tree, date2dist=self.date2dist,
+                                logger=self.logger, n_branches_posterior=n_branches_posterior)
 
         if Tc=='skyline': # restrict skyline model optimization to last iteration
             self.merger_model.optimize_skyline(**kwargs)
@@ -737,7 +789,6 @@ class TreeTime(ClockTree):
                 except:
                     self.logger("setting of coalescent time scale failed", 1, warn=True)
 
-        self.merger_model.attach_to_tree()
 
 
     def relaxed_clock(self, slack=None, coupling=None, **kwargs):
@@ -827,7 +878,7 @@ class TreeTime(ClockTree):
             'seq_mode'   : ('marginal' if sequence_marginal else 'joint') if self.aln else 'no sequences given',
             'seq_LH'     : (self.tree.sequence_marginal_LH if sequence_marginal else self.tree.sequence_joint_LH) if self.aln else 0,
             'pos_mode'   : 'marginal' if time_marginal else 'joint',
-            'pos_LH'     : self.tree.positional_marginal_LH if time_marginal else self.tree.positional_joint_LH,
+            'pos_LH'     : self.tree.positional_LH,
             'coal_mode'  : Tc,
             'coal_LH'    : self.tree.coalescent_joint_LH,
         }
@@ -873,7 +924,7 @@ class TreeTime(ClockTree):
             n.branch_length=n.mutation_length
         self.logger("TreeTime._find_best_root: searching for the best root position...",2)
         Treg = self.setup_TreeRegression(covariation=covariation)
-        return Treg.optimal_reroot(force_positive=force_positive, slope=slope)['node']
+        return Treg.optimal_reroot(force_positive=force_positive, slope=slope, keep_node_order=self.keep_node_order)['node']
 
 
 def plot_vs_years(tt, step = None, ax=None, confidence=None, ticks=True, **kwargs):
