@@ -1,12 +1,7 @@
-from __future__ import division, print_function, absolute_import
 import os,sys
 import datetime
 import pandas as pd
 import numpy as np
-from scipy.interpolate import interp1d
-from scipy.integrate import quad
-from scipy import stats
-from scipy.ndimage import binary_dilation
 from . import TreeTimeError
 
 class DateConversion(object):
@@ -438,7 +433,9 @@ def tree_inference(aln_fname, tree_fname, tmp_dir=None,
             continue
     os.chdir(cwd)
     if T is None:
-        print("tree building failed. tried", ", ".join(methods), "but none worked")
+        msg = f"tree building failed. tried '{','.join(methods)}', but none worked"
+        print(msg)
+        raise(TreeTimeError(msg))
     else:
         Phylo.write(T, tree_fname, 'newick')
 
